<?php

class Controller_Api_V0_StudyPlan extends Controller_Handler_Api {
    private function normalizeDisciplinesData($discipline, $facultyID) {
        if (!isset($discipline->subjectID) && isset($discipline->externalID)) {
            $discipline->subjectID = Model_Subject::withExternalID($discipline->externalID, $discipline->name, '', $facultyID);
        }

        foreach ($discipline->teachers as $teacher) {
            if (empty($teacher->hashSnils)) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline with empty teacher\'s hash INILA',
                    '{2}' => json_encode($discipline),
                ));
            } else {
                $ID = Model_Teacher::withINILA($teacher->hashSnils);
                if ($ID != -1) {
                    $discipline->teacherIDs[] = $ID;
                }
            }
        }

        if (empty($discipline->teacherIDs)) {
            if (!empty($discipline->teachers)) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline without teachers',
                    '{2}' => json_encode($discipline),
                ));
            }
        }

        switch ($discipline->type) {
            case 'Экзамен':
                $discipline->type = Model_Discipline::EXAM;
                break;

            case 'Зачет':
                $discipline->type = Model_Discipline::CREDIT;
                break;

            default:
                throw new InvalidArgumentException('Discipline has bad type');
        }

        return $discipline;
    }

    private function processStudyPlan($studyPlan, $year, $facultyID) {
        if (!isset($studyPlan->id) && isset($studyPlan->externalID)) {
            $ID = Model_Plan::withExternalID($studyPlan->externalID);
            if ($ID == -1) {
                $studyPlan->planModel = Model_Plan::make()
                    ->externalID($studyPlan->externalID)
                    ->year($year)
                    ->faculty($facultyID)
                    ->create();
            } else {
                $studyPlan->planModel = Model_Plan::load($ID);
            }
        }

        foreach ($studyPlan->disciplines as &$discipline) {
            try {
                $discipline = $this->normalizeDisciplinesData($discipline, $facultyID);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($discipline),
                ));
                $discipline = null;
            }
        }

        return $studyPlan;
    }

    private function processDiscipline($discipline, $semesterID, $facultyID) {
        if (isset($discipline->subjectID)) {
            $disciplineModel = Model_Discipline::find($facultyID, $semesterID, $discipline->subjectID, $discipline->type);
        }

        $lTeacherIDs = [];
        if (!empty($disciplineModel)) {
            $disciplineModel->update();

            // Сравнить список пришедших преподавателей и тех, которые уже прикреплены
            if (!empty($discipline->teacherIDs)) {
                $lTeacherIDs = array_diff($discipline->teacherIDs, $disciplineModel->getTeachers()->as_array());
            }
        } else {
            if (empty($discipline->teacherIDs)) {
                throw new InvalidArgumentException('Cannot create discipline without teachers');
            }

            $disciplineModel = Model_Discipline::make()
                ->author($discipline->teacherIDs[0])
                ->subject($discipline->subjectID)
                ->semester($semesterID)
                ->type($discipline->type)
                ->faculty($facultyID)
                ->create();

            $lTeacher = $disciplineModel->getTeachers()->as_array();
            foreach ($lTeacher as $teacher) {
                $lTeacherIDs[] = $teacher['ID'];
            }
        }

        foreach ($lTeacherIDs as $teacherID) {
            $teacher = Model_Teacher::with($teacherID);
            $disciplineModel->bind($teacher);
        }

        return (int)$disciplineModel->ID;
    }

    private function processDisciplines($studyPlan, $semesterID, $facultyID) {
        $res = [];

        foreach ($studyPlan->disciplines as $discipline) {
            if ($discipline == null) continue;

            try {
                $res[] = $this->processDiscipline($discipline, $semesterID, $facultyID);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($discipline),
                ));
            }
        }

        return $res;
    }

//    /**
//     * @api {put} api/v0/studyPlan Add new study plan(s)
//     * @apiName Add new study plan
//     * @apiGroup Study plans
//     * @apiVersion 0.1.2
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} externalId
//     * @apiParam {Array} disciplines
//     */
    public function action_put_index() {
        $res = [];

        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());

                $year = (int)$data->year;
                $semesterID = (int)Model_Semesters::find($year, (int)$data->semester)->ID;
                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

                foreach ($data->plans as $item) {
                    $planData = $this->processStudyPlan($item, $year, $facultyID);
                    $res[] = $this->processDisciplines($planData, $semesterID, $facultyID);
                }
            } else {
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
